@@ -3,4 +3,9 @@
 
 from ._detector import DetectorBackendPluginBase, DetectorPlotPluginBase, DetectorViewPluginBase
 from ._processor_main import processor_main
-from .processor import ProcessorBackendPluginBase, ProcessorPlotPluginBase, ProcessorViewPluginBase
+from .processor import (
+    ProcessorBackendPluginBase,
+    ProcessorPlotPluginBase,
+    ProcessorPluginSpec,
+    ProcessorViewPluginBase,
+)
