@@ -15,7 +15,13 @@ import numpy.typing as npt
 
 from acconeer.exptool import a121
 from acconeer.exptool.a121._h5_utils import _create_h5_string_dataset
-from acconeer.exptool.a121.algo import AlgoConfigBase
+from acconeer.exptool.a121.algo import (
+    APPROX_BASE_STEP_LENGTH_M,
+    ENVELOPE_FWHM_M,
+    AlgoConfigBase,
+    get_distance_filter_edge_margin,
+    select_prf,
+)
 
 from ._aggregator import (
     Aggregator,
@@ -227,12 +233,6 @@ class Detector:
 
     MIN_DIST_M = 0.0
     MAX_DIST_M = 17.0
-    MAX_MEASURABLE_DIST_M = {
-        a121.PRF.PRF_19_5_MHz: 3.1,
-        a121.PRF.PRF_13_0_MHz: 7.0,
-        a121.PRF.PRF_8_7_MHz: 12.7,
-        a121.PRF.PRF_6_5_MHz: 18.5,
-    }
     MIN_LEAKAGE_FREE_DIST_M = {
         a121.Profile.PROFILE_1: 0.12,
         a121.Profile.PROFILE_2: 0.28,
@@ -504,7 +504,7 @@ class Detector:
 
     @classmethod
     def _valid_detector_config_range(cls, config: DetectorConfig) -> bool:
-        return cls.MIN_DIST_M < config.start_m and config.end_m < cls.MAX_DIST_M
+        return cls.MIN_DIST_M <= config.start_m and config.end_m <= cls.MAX_DIST_M
 
     @staticmethod
     def _close_range_calibrated(context: DetectorContext) -> bool:
@@ -554,7 +554,7 @@ class Detector:
         status = self.get_detector_status(self.detector_config, self.context, self.sensor_id)
 
         if not status.ready_to_start:
-            raise RuntimeError("Not ready to start")
+            raise RuntimeError(f"Not ready to start ({status.detector_state.name})")
 
         if not skip_calibration:
             self.calibrate_noise()
@@ -881,7 +881,7 @@ class Detector:
                 cfar_margin_m = (
                     Processor.calc_cfar_margin(profile, step_length)
                     * step_length
-                    * Processor.APPROX_BASE_STEP_LENGTH_M
+                    * APPROX_BASE_STEP_LENGTH_M
                 )
                 min_dist_m[profile] += cfar_margin_m
 
@@ -900,7 +900,7 @@ class Detector:
         for idx in range(len(breakpoints) - 1):
             processing_gain = Aggregator.calc_processing_gain(profile, step_length)
             subsweep_end_point_m = max(
-                Processor.APPROX_BASE_STEP_LENGTH_M * breakpoints[idx + 1],
+                APPROX_BASE_STEP_LENGTH_M * breakpoints[idx + 1],
                 cls.HWAAS_MIN_DISTANCE,
             )
             rlg = signal_quality + 40 * np.log10(subsweep_end_point_m) - np.log10(processing_gain)
@@ -925,7 +925,7 @@ class Detector:
         2. Add an additional margin to segments with neighbouring segments for segment overlap
         """
 
-        margin_p = Processor.distance_filter_edge_margin(profile, step_length) * step_length
+        margin_p = get_distance_filter_edge_margin(profile, step_length) * step_length
         left_margin = margin_p
         right_margin = margin_p
 
@@ -961,7 +961,7 @@ class Detector:
         closest, but not longer than the limit.
         """
 
-        fwhm_p = Processor.ENVELOPE_FWHM_M[profile] / Processor.APPROX_BASE_STEP_LENGTH_M
+        fwhm_p = ENVELOPE_FWHM_M[profile] / APPROX_BASE_STEP_LENGTH_M
         limit = int(fwhm_p / cls.MIN_NUM_POINTS_IN_ENVELOPE_FWHM_SPAN)
 
         if user_limit is not None:
@@ -1001,7 +1001,7 @@ class Detector:
                 hwaas=plan.hwaas[0],
                 receiver_gain=5,
                 phase_enhancement=True,
-                prf=cls._select_prf(plan.breakpoints[1], plan.profile),
+                prf=select_prf(plan.breakpoints[1], plan.profile),
             )
         )
         return a121.SensorConfig(subsweeps=subsweeps, sweeps_per_frame=10)
@@ -1028,7 +1028,7 @@ class Detector:
                         hwaas=plan.hwaas[bp_idx],
                         receiver_gain=10,
                         phase_enhancement=True,
-                        prf=cls._select_prf(plan.breakpoints[bp_idx + 1], plan.profile),
+                        prf=select_prf(plan.breakpoints[bp_idx + 1], plan.profile),
                     )
                 )
                 subsweep_indexes.append(subsweep_idx)
@@ -1039,28 +1039,11 @@ class Detector:
             processor_specs_subsweep_indexes,
         )
 
-    @classmethod
-    def _select_prf(cls, breakpoint: int, profile: a121.Profile) -> a121.PRF:
-        max_meas_dist_m = copy.copy(cls.MAX_MEASURABLE_DIST_M)
-
-        if (
-            a121.PRF.PRF_19_5_MHz in max_meas_dist_m
-            and profile != a121.Profile.PROFILE_1
-            and profile != a121.Profile.PROFILE_2
-        ):
-            del max_meas_dist_m[a121.PRF.PRF_19_5_MHz]
-
-        breakpoint_m = breakpoint * Processor.APPROX_BASE_STEP_LENGTH_M
-        viable_prfs = [
-            prf for prf, max_dist_m in max_meas_dist_m.items() if breakpoint_m < max_dist_m
-        ]
-        return sorted(viable_prfs, key=lambda prf: prf.frequency)[-1]
-
     @classmethod
     def _m_to_points(cls, breakpoints_m: list[float], step_length: int) -> list[int]:
         bpts_m = np.array(breakpoints_m)
-        start_point = int(bpts_m[0] / Processor.APPROX_BASE_STEP_LENGTH_M)
-        num_steps = (bpts_m[-1] - bpts_m[0]) / (Processor.APPROX_BASE_STEP_LENGTH_M)
+        start_point = int(bpts_m[0] / APPROX_BASE_STEP_LENGTH_M)
+        num_steps = (bpts_m[-1] - bpts_m[0]) / (APPROX_BASE_STEP_LENGTH_M)
         bpts = num_steps / (bpts_m[-1] - bpts_m[0]) * (bpts_m - bpts_m[0]) + start_point
         return [(bpt // step_length) * step_length for bpt in bpts]
 
