@@ -179,6 +179,7 @@ class BackendPlugin(DetectorBackendPluginBase[SharedState]):
         algo_group = self._opened_record.get_algo_group(self.key)
         _, config = _load_algo_data(algo_group)
         self.shared_state.config = config
+        self.shared_state.sensor_id = r.sensor_id
 
     @is_task
     def start_session(self, *, with_recorder: bool = True) -> None:
@@ -471,7 +472,7 @@ class PlotPlugin(DetectorPlotPluginBase):
         pen = pg.mkPen("k", width=1)
         span_deg = 25
         for r in np.flip(np.arange(self.num_sectors) + 1):
-            sector = pg.QtGui.QGraphicsEllipseItem(-r, -r, r * 2, r * 2)
+            sector = pg.QtWidgets.QGraphicsEllipseItem(-r, -r, r * 2, r * 2)
             sector.setStartAngle(-16 * span_deg)
             sector.setSpanAngle(16 * span_deg * 2)
             sector.setPen(pen)
